import multiprocessing
import sys
from unittest import skipUnless


def requires_python_version(version):
    version_tuple = tuple(map(int, str(version).split('.')))
    return skipUnless(sys.version_info >= version_tuple,
                      'Requires python version %s' % version)


class SharedCounter(object):
    def __init__(self):
        self._val = multiprocessing.Value('i', 0)

    def increment(self, n=1):
        with self._val.get_lock():
            self._val.value += n
            return self._val.value

    @property
    def value(self):
        return self._val.value



import os
from cheap_repr import repr_str, cheap_repr
from birdseye import eye

path = os.path.join(os.path.expanduser('~'), '.birdseye_test.db')

if not os.environ.get('BIRDSEYE_SERVER_RUNNING'):
    # Remove the database to start from scratch
    if os.path.exists(path):
        os.remove(path)

os.environ.setdefault('BIRDSEYE_DB', 'sqlite:///' + path)

repr_str.maxparts = 30
cheap_repr.raise_exceptions = True

eye.num_samples['big']['list'] = 10



#!/usr/bin/env python3

"""
This script parses output from mypy and makes it more manageable, particularly
if lots of warnings are raised that you want to ignore. It's an alternative to
'# type: ignore' comments and other ways of appeasing mypy that doesn't
interfere with your source code.

Here is how to use it in this project:

python3 -m mypy -p birdseye --ignore-missing-imports | misc/mypy_filter.py misc/mypy_ignore.txt

This will output all warning messages not found in mypy_ignore.txt. It will also group them
so that you don't have to read the same message twice. Inspect the output.
If it contains any legitimate errors, or messages that are generic enough to apply to other
situations, fix the code to remove them. Once the output looks safe, run the command again
with 'ok' at the end, i.e:

python3 -m mypy -p birdseye --ignore-missing-imports | misc/mypy_filter.py misc/mypy_ignore.txt ok

This will add any remaining warnings to mypy_ignore.txt so that they are ignored in the future.
"""

import sys
import re
from collections import defaultdict


def main():
    ignore_messages = []
    ignore_file = None
    if len(sys.argv) > 1:
        ignore_file = sys.argv[1]
        with open(ignore_file) as f:
            ignore_messages = f.readlines()

    messages = defaultdict(lambda: defaultdict(set))
    for line in sys.stdin:
        match = re.match(r'^(.+?):(\d+): (.+)$', line)
        if not match or any(ignore in line for ignore in ignore_messages):
            continue
        path, lineno, message = match.groups()
        messages[message][path].add(int(lineno))

    if sys.argv[2:3] == ['ok']:
        with open(ignore_file, 'a') as f:
            for message in sorted(messages):
                f.write(message + '\n')
        print('Added', len(messages), 'messages to', ignore_file)
    else:
        for message, places in sorted(messages.items()):
            print(message)
            for path, linenos in sorted(places.items()):
                print(' ', path, ':', ', '.join(map(str, sorted(linenos))))
            print()


main()



# coding=utf8

import ast
import linecache
import unittest
from tempfile import mkstemp

import asttokens
import numpy as np
import pandas as pd
from cheap_repr import cheap_repr

from birdseye.utils import common_ancestor, short_path, flatten_list, is_lambda, source_without_decorators, PY3, \
    read_source_file


def def_decorator(_):
    def actual_decorator(f):
        return f

    return actual_decorator


def eye(f):
    return f


@def_decorator('def')
@eye
def define(defx, defy):
    """
def def def

@eye
def define(defx, defy):
    """
    def inner():
        pass
    return defx + defy + inner


define_source = '''\
def define(defx, defy):
    """
def def def

@eye
def define(defx, defy):
    """
    def inner():
        pass
    return defx + defy + inner'''


class TestUtils(unittest.TestCase):
    def test_common_ancestor(self):
        self.assertEqual(
            common_ancestor(['/123/456', '/123/457', '/123/abc/def']),
            '/123/'
        )
        self.assertEqual(
            common_ancestor(['\\123\\456', '\\123\\457', '\\123\\abc\\def']),
            '\\123\\'
        )

    def test_short_path(self):
        def check(paths, result):
            self.assertEqual(result, [short_path(path, paths) for path in paths])

        check(['/123/456', '/123/457', '/123/abc/def'], ['456', '457', 'abc/def'])
        check(['/123/456'], ['456'])
        check(['/123'], ['/123'])
        check(['/123/456', '/abc/def'], ['/123/456', '/abc/def'])
        check(['\\123\\456', '\\123\\457', '\\123\\abc\\def'], ['456', '457', 'abc\\def'])
        check(['\\123\\456'], ['456'])
        check(['\\123'], ['\\123'])
        check(['\\123\\456', '\\abc\\def'], ['\\123\\456', '\\abc\\def'])

    def test_flatten_list(self):
        def check(inp, out):
            self.assertEqual(flatten_list(inp), out)

        check([], [])
        check(['abc'], ['abc'])
        check(['ab', 'cd'], ['ab', 'cd'])
        check(['ab', ['cd', 'ef']], ['ab', 'cd', 'ef'])
        check([['x', 'y'], 'ab', [['0', '1'], [[], 'cd', 'ef', ['ghi', [[['jkl']]]]]]],
              ['x', 'y', 'ab', '0', '1', 'cd', 'ef', 'ghi', 'jkl'])

    def test_is_lambda(self):
        self.assertTrue(is_lambda(lambda: 0))
        self.assertTrue(is_lambda(lambda x, y: x + y))
        self.assertFalse(is_lambda(min))
        self.assertFalse(is_lambda(flatten_list))
        self.assertFalse(is_lambda(self.test_is_lambda))

    def test_open_with_encoding_check(self):
        filename = mkstemp()[1]

        def write(stuff):
            with open(filename, 'wb') as f:
                f.write(stuff)
            linecache.cache.pop(filename, None)

        def read():
            return read_source_file(filename).strip()

        # Correctly declared encodings

        write(u'# coding=utf8\né'.encode('utf8'))
        self.assertEqual(u'é', read())

        write(u'# coding=gbk\né'.encode('gbk'))
        self.assertEqual(u'é', read())

        # Wrong encodings

        write(u'# coding=utf8\né'.encode('gbk'))
        self.assertRaises(UnicodeDecodeError, read)

        write(u'# coding=gbk\né'.encode('utf8'))
        self.assertFalse(u'é' in read())

        # In Python 3 the default encoding is assumed to be UTF8
        if PY3:
            write(u'é'.encode('utf8'))
            self.assertEqual(u'é', read())

            write(u'é'.encode('gbk'))

            # The lack of an encoding when one is needed
            # ultimately raises a SyntaxError
            self.assertRaises(SyntaxError, read)

    def test_source_without_decorators(self):
        source = read_source_file(__file__)
        tokens = asttokens.ASTTokens(source, parse=True)
        function_def_node = next(n for n in ast.walk(tokens.tree)
                                 if isinstance(n, ast.FunctionDef) and
                                 n.name == 'define')
        self.assertEqual(define_source,
                         source_without_decorators(tokens, function_def_node)[1])

    def test_cheap_repr(self):
        arr = np.arange(10000)
        arr = arr.reshape((100, 100))
        df = pd.DataFrame(arr)
        series = df[0]
        self.assertEqual(cheap_repr(series), "0 = 0; 1 = 100; 2 = 200; ...; 97 = 9700; 98 = 9800; 99 = 9900")

    def test_read_source_file_as_string(self):
        import linecache
        import birdseye

        source = """
from birdseye import eye

@eye
def func():
    return 42
"""
        filename = "<some-non-filename-string>"
        co = compile(source, filename, "exec")
        linecache.cache[filename] = (
            len(source),
            None,
            [line + '\n' for line in source.splitlines()],
            filename,
        )
        exec(co, {"birdseye": birdseye})
        rv = read_source_file(filename)
        self.assertEqual(rv, source)
        return


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from threading import Thread
from time import sleep

import requests
from littleutils import only
from selenium import webdriver
from selenium.webdriver import ActionChains
from selenium.webdriver.chrome.options import Options

from birdseye import eye
from birdseye.server import app


@eye
def foo():
    for i in range(20):
        for j in range(3):
            int(i * 13 + j * 17)
            if i > 0:
                try:
                    assert j
                except AssertionError:
                    pass
    str(bar())

    x = list(range(1, 30, 2))
    list(x)


@eye
def bar():
    pass


class TestInterface(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        chrome_options.add_argument("--no-sandbox")
        self.driver = webdriver.Chrome(options=chrome_options)
        self.driver.set_window_size(1400, 1000)
        self.driver.implicitly_wait(2)
        if not os.environ.get('BIRDSEYE_SERVER_RUNNING'):
            Thread(target=lambda: app.run(port=7777)).start()

    def test(self):
        try:
            self._do_test()
        except:
            self.driver.save_screenshot('error_screenshot.png')
            raise

    def _do_test(self):
        foo()
        driver = self.driver

        # On the index page, note the links to the function and call
        driver.get('http://localhost:7777/')
        function_link = driver.find_element_by_link_text('foo')
        function_url = function_link.get_attribute('href')
        call_url = function_link.find_element_by_xpath('..//i/..').get_attribute('href')

        # On the file page, check that the links still match
        driver.find_element_by_partial_link_text('test_interface').click()
        function_link = driver.find_element_by_link_text('foo')
        self.assertEqual(function_link.get_attribute('href'), function_url)
        self.assertEqual(call_url, function_link.find_element_by_xpath('..//i/..').get_attribute('href'))

        # Finally navigate to the call and check the original call_url
        function_link.click()
        driver.find_element_by_css_selector('table a').click()
        self.assertEqual(driver.current_url, call_url)

        # Test hovering, clicking on expressions, and stepping through loops

        vals = {'i': 0, 'j': 0}
        exprs = driver.find_elements_by_class_name('has_value')
        expr_value = driver.find_element_by_id('box_value')

        expr_strings = [
            'i * 13 + j * 17',
            'j * 17',
            'i * 13',
        ]

        def find_by_text(text, elements):
            return only(n for n in elements if n.text == text)

        def find_expr(text):
            return find_by_text(text, exprs)

        def tree_nodes(root=driver):
            return root.find_elements_by_class_name('jstree-node')

        def select(node, prefix, value_text):
            self.assertIn('box', classes(node))
            self.assertIn('has_value', classes(node))
            self.assertNotIn('selected', classes(node))
            node.click()
            self.assertIn('selected', classes(node))
            self.assertEqual(expr_value.text, value_text)
            tree_node = tree_nodes()[-1]
            self.assertEqual(tree_node.text, prefix + value_text)
            return tree_node

        def classes(node):
            return set(node.get_attribute('class').split())

        def assert_classes(node, *cls):
            self.assertEqual(classes(node), set(cls))

        for i, expr in enumerate(expr_strings):
            find_expr(expr).click()

        def step(loop, increment):
            selector = '.loop-navigator > .btn:%s-child' % ('first' if increment == -1 else 'last')
            buttons = driver.find_elements_by_css_selector(selector)
            self.assertEqual(len(buttons), 2)
            buttons[loop].click()
            vals['ij'[loop]] += increment

            for expr in expr_strings:
                ActionChains(driver).move_to_element(find_expr(expr)).perform()
                value = str(eval(expr, {}, vals))
                self.assertEqual(expr_value.text, value)
                node = only(n for n in tree_nodes()
                            if n.text.startswith(expr + ' ='))
                self.assertEqual(node.text, '%s = int: %s' % (expr, value))

        stmt = find_by_text('assert j', driver.find_elements_by_class_name('stmt'))
        assert_classes(stmt, 'stmt', 'stmt_uncovered', 'box')

        step(0, 1)
        select(stmt, 'assert j : ', 'AssertionError')
        assert_classes(stmt, 'stmt', 'selected', 'box', 'hovering', 'has_value', 'exception_node')
        step(1, 1)
        self.assertEqual(tree_nodes()[-1].text, 'assert j : fine')
        assert_classes(stmt, 'stmt', 'selected', 'box', 'has_value', 'value_none')
        step(1, 1)
        step(0, -1)
        self.assertTrue({'stmt', 'stmt_uncovered', 'selected', 'box'} <= classes(stmt))
        step(1, -1)

        # Expanding values
        x_node = find_expr('x')
        tree_node = select(x_node, 'x = list: ', '[1, 3, 5, ..., 25, 27, 29]')
        tree_node.find_element_by_class_name('jstree-ocl').click()  # expand
        sleep(0.2)
        self.assertEqual([n.text for n in tree_nodes(tree_node)],
                         ['len() = 15',
                          '0 = int: 1',
                          '1 = int: 3',
                          '2 = int: 5',
                          '3 = int: 7',
                          '4 = int: 9',
                          '10 = int: 21',
                          '11 = int: 23',
                          '12 = int: 25',
                          '13 = int: 27',
                          '14 = int: 29']),

        # Click on an inner call
        find_expr('bar()').find_element_by_class_name('inner-call').click()
        self.assertEqual(driver.find_element_by_tag_name('h2').text,
                         'Call to function: bar')

    def tearDown(self):
        if not os.environ.get('BIRDSEYE_SERVER_RUNNING'):
            self.assertEqual(requests.post('http://localhost:7777/kill').text,
                             'Server shutting down...')



"""
This script lets you view a screenshot of the browser produced by selenium
when a test fails on travis.

To run this script you will first need to:

    pip install pyperclip jsonfinder requests

Open a travis build job with a failed interface test. There should be logs with a big JSON
blob produced by selenium, including an encoding of a screenshot of the browswer when the
error occurred. Copy the URL of the job to your clipboard. Here is an example:

    https://travis-ci.org/alexmojaki/birdseye/jobs/290114752

Then run the script as simply follows:

    python travis_screenshot.py

The script doesn't take arguments, it reads the URL from the clipboard.

The result will be a file such as screen_0.png.

"""

import json
import re
from base64 import b64decode

import pyperclip
import requests
from jsonfinder import jsonfinder

log_url = re.sub(r'.+/(jobs/\d+)', r'https://api.travis-ci.org/\1/log.txt?deansi=true',
                 pyperclip.paste())

text = requests.get(log_url).text

i = 0
for _, _, obj in jsonfinder(text):
    if (isinstance(obj, dict) and
                'selenium' in str(obj) and
                'screen' in obj.get('value', {})):
        obj = obj['value']
        screen = obj.pop('screen')
        message = obj
        try:
            message = message['message']
            message = json.loads(message)
            message = message['errorMessage']
        except (ValueError, KeyError):
            pass
        filename = 'screen_%s.png' % i
        i += 1
        with open(filename, 'wb') as f:
            f.write(b64decode(screen))
            print('Wrote ' + filename)
            print('Message: %s' % message)
